@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-
 import stea
 import argparse
 from semeio import valid_file
@@ -30,15 +28,11 @@ def _get_args_parser():
     return parser
 
 
-def main_entry_point(args=None):
+def main_entry_point():
     parser = _get_args_parser()
-    options = parser.parse_args(args)
+    options = parser.parse_args()
     stea_input = stea.SteaInput([options.config])
     res = stea.calculate(stea_input)
     for res, value in res.results(stea.SteaKeys.CORPORATE).items():
         with open("{}_0".format(res), "w") as ofh:
             ofh.write("{}\n".format(value))
-
-
-if __name__ == "__main__":
-    main_entry_point()
