@@ -2,24 +2,25 @@ import json
 import logging
 import os
 import pytest
-import sys
+from threading import Thread
 
-from semeio.communication import SemeioScript, SEMEIOSCRIPT_LOG_FILE
+from semeio.communication import SEMEIOSCRIPT_LOG_FILE, SemeioScript
 
-if sys.version_info >= (3, 3):
-    from unittest.mock import Mock
-else:
-    from mock import Mock
+from unittest.mock import Mock
+
+
+def _ert_mock(config_path="config_path", user_config_file="config_file"):
+    resconfig_mock = Mock()
+    resconfig_mock.config_path = config_path
+    resconfig_mock.user_config_file = user_config_file
+
+    return Mock(resConfig=Mock(return_value=resconfig_mock))
 
 
 def test_semeio_script_publish(tmpdir):
     tmpdir.chdir()
-    enspath = "enspath"
-    ert = Mock(
-        resConfig=Mock(
-            return_value=Mock(model_config=Mock(getEnspath=Mock(return_value=enspath)))
-        )
-    )
+    config_path, user_config_file = "config_path", "config_file"
+    ert = _ert_mock(config_path, user_config_file)
 
     namespace = "arbitrary_data"
     data = [1, 2, 3, 4, "This is a very important string"]
@@ -32,23 +33,23 @@ def test_semeio_script_publish(tmpdir):
     my_super_script.run()
 
     expected_outputfile = os.path.join(
-        enspath, "reports", MySuperScript.__name__, namespace + ".json"
+        config_path,
+        "reports",
+        user_config_file,
+        MySuperScript.__name__,
+        namespace + ".json",
     )
 
     with open(expected_outputfile) as f:
         published_data = json.load(f)
 
-    assert data == published_data
+    assert [data] == published_data
 
 
 def test_semeio_script_logging(tmpdir):
     tmpdir.chdir()
-    enspath = "enspath"
-    ert = Mock(
-        resConfig=Mock(
-            return_value=Mock(model_config=Mock(getEnspath=Mock(return_value=enspath)))
-        )
-    )
+    config_path, user_config_file = "config_path", "config_file"
+    ert = _ert_mock(config_path, user_config_file)
 
     msg = "My logging msg"
 
@@ -60,7 +61,11 @@ def test_semeio_script_logging(tmpdir):
     my_super_script.run()
 
     expected_logfile = os.path.join(
-        enspath, "reports", MySuperScript.__name__, SEMEIOSCRIPT_LOG_FILE
+        config_path,
+        "reports",
+        user_config_file,
+        MySuperScript.__name__,
+        SEMEIOSCRIPT_LOG_FILE,
     )
 
     with open(expected_logfile) as f:
@@ -89,12 +94,8 @@ def assert_log(messages, log_file):
 )
 def test_semeio_script_multiple_logging(messages, tmpdir):
     tmpdir.chdir()
-    enspath = "enspath"
-    ert = Mock(
-        resConfig=Mock(
-            return_value=Mock(model_config=Mock(getEnspath=Mock(return_value=enspath)))
-        )
-    )
+    config_path, user_config_file = "config_path", "config_file"
+    ert = _ert_mock(config_path, user_config_file)
 
     class MySuperScript(SemeioScript):
         def run(self, *args):
@@ -106,7 +107,11 @@ def test_semeio_script_multiple_logging(messages, tmpdir):
                 assert_log(posted_messages, expected_logfile)
 
     expected_logfile = os.path.join(
-        enspath, "reports", MySuperScript.__name__, SEMEIOSCRIPT_LOG_FILE
+        config_path,
+        "reports",
+        user_config_file,
+        MySuperScript.__name__,
+        SEMEIOSCRIPT_LOG_FILE,
     )
 
     my_super_script = MySuperScript(ert)
@@ -117,12 +122,8 @@ def test_semeio_script_multiple_logging(messages, tmpdir):
 
 def test_semeio_script_post_logging(tmpdir):
     tmpdir.chdir()
-    enspath = "enspath"
-    ert = Mock(
-        resConfig=Mock(
-            return_value=Mock(model_config=Mock(getEnspath=Mock(return_value=enspath)))
-        )
-    )
+    config_path, user_config_file = "config_path", "config_file"
+    ert = _ert_mock(config_path, user_config_file)
 
     class MySuperScript(SemeioScript):
         def run(self, *args):
@@ -134,7 +135,11 @@ def test_semeio_script_post_logging(tmpdir):
     logging.error("A second message - not from MySuperScript")
 
     expected_logfile = os.path.join(
-        enspath, "reports", MySuperScript.__name__, SEMEIOSCRIPT_LOG_FILE
+        config_path,
+        "reports",
+        user_config_file,
+        MySuperScript.__name__,
+        SEMEIOSCRIPT_LOG_FILE,
     )
 
     with open(expected_logfile) as f:
@@ -145,12 +150,8 @@ def test_semeio_script_post_logging(tmpdir):
 
 def test_semeio_script_pre_logging(tmpdir):
     tmpdir.chdir()
-    enspath = "enspath"
-    ert = Mock(
-        resConfig=Mock(
-            return_value=Mock(model_config=Mock(getEnspath=Mock(return_value=enspath)))
-        )
-    )
+    config_path, user_config_file = "config_path", "config_file"
+    ert = _ert_mock(config_path, user_config_file)
 
     class MySuperScript(SemeioScript):
         def run(self, *args):
@@ -163,7 +164,39 @@ def test_semeio_script_pre_logging(tmpdir):
     my_super_script.run()
 
     expected_logfile = os.path.join(
-        enspath, "reports", MySuperScript.__name__, SEMEIOSCRIPT_LOG_FILE
+        config_path,
+        "reports",
+        user_config_file,
+        MySuperScript.__name__,
+        SEMEIOSCRIPT_LOG_FILE,
+    )
+
+    with open(expected_logfile) as f:
+        loaded_log = f.readlines()
+
+    assert len(loaded_log) == 1
+
+
+def test_semeio_script_concurrent_logging(tmpdir):
+    tmpdir.chdir()
+    config_path, user_config_file = "config_path", "config_file"
+    ert = _ert_mock(config_path, user_config_file)
+
+    class MySuperScript(SemeioScript):
+        def run(self, *args):
+            logging.error("A message from MySuperScript")
+            thread = Thread(target=lambda: logging.error("External event."))
+            thread.start()
+            thread.join()
+
+    MySuperScript(ert).run()
+
+    expected_logfile = os.path.join(
+        config_path,
+        "reports",
+        user_config_file,
+        MySuperScript.__name__,
+        SEMEIOSCRIPT_LOG_FILE,
     )
 
     with open(expected_logfile) as f:
@@ -174,12 +207,8 @@ def test_semeio_script_pre_logging(tmpdir):
 
 def test_semeio_script_post_logging_exception(tmpdir):
     tmpdir.chdir()
-    enspath = "enspath"
-    ert = Mock(
-        resConfig=Mock(
-            return_value=Mock(model_config=Mock(getEnspath=Mock(return_value=enspath)))
-        )
-    )
+    config_path, user_config_file = "config_path", "config_file"
+    ert = _ert_mock(config_path, user_config_file)
 
     class MySuperScript(SemeioScript):
         def run(self, *args):
@@ -195,7 +224,11 @@ def test_semeio_script_post_logging_exception(tmpdir):
     logging.error("A second message - not from MySuperScript")
 
     expected_logfile = os.path.join(
-        enspath, "reports", MySuperScript.__name__, SEMEIOSCRIPT_LOG_FILE
+        config_path,
+        "reports",
+        user_config_file,
+        MySuperScript.__name__,
+        SEMEIOSCRIPT_LOG_FILE,
     )
 
     with open(expected_logfile) as f:
@@ -206,12 +239,8 @@ def test_semeio_script_post_logging_exception(tmpdir):
 
 def test_semeio_script_keyword_args(tmpdir):
     tmpdir.chdir()
-    enspath = "enspath"
-    ert = Mock(
-        resConfig=Mock(
-            return_value=Mock(model_config=Mock(getEnspath=Mock(return_value=enspath)))
-        )
-    )
+    config_path, user_config_file = "config_path", "config_file"
+    ert = _ert_mock(config_path, user_config_file)
 
     class MySuperScript(SemeioScript):
         def run(self, param_A, param_B):
@@ -222,7 +251,11 @@ def test_semeio_script_keyword_args(tmpdir):
     my_super_script.run(param_B="param_B", param_A="param_A")
 
     expected_outputfile = os.path.join(
-        enspath, "reports", MySuperScript.__name__, SEMEIOSCRIPT_LOG_FILE
+        config_path,
+        "reports",
+        user_config_file,
+        MySuperScript.__name__,
+        SEMEIOSCRIPT_LOG_FILE,
     )
 
     with open(expected_outputfile) as f:
