@@ -1,6 +1,7 @@
 import filecmp
 import logging
 import os
+from packaging import version
 from distutils.dir_util import copy_tree
 
 import pandas as pd
@@ -133,9 +134,7 @@ def test_runs_different_reals_all_ok(realization_id):
     design2params.run(
         realization_id,
         "design_matrix_tampered_defaults.xlsx",
-        "DesignSheet01",
-        "DefaultValues",
-        design2params._PARAMETERS_TXT,
+        parametersfilename=design2params._PARAMETERS_TXT,
         log_level=logging.DEBUG,
     )
 
@@ -147,14 +146,7 @@ def test_runs_different_reals_all_ok(realization_id):
 def test_empty_defaults(tmpdir):
     # pylint: disable=abstract-class-instantiated
     tmpdir.chdir()
-    designsheet_df = pd.DataFrame()
-    defaultssheet_df = pd.DataFrame()
-    writer = pd.ExcelWriter("design_matrix.xlsx")
-    designsheet_df.to_excel(writer, sheet_name="DesignSheet01", index=False)
-    defaultssheet_df.to_excel(
-        writer, sheet_name="DefaultValues", index=False, header=None
-    )
-    writer.save()
+    write_design_xlsx("design_matrix.xlsx")
 
     parsed_defaults = design2params._read_defaultssheet(
         "design_matrix.xlsx", "DefaultValues"
@@ -164,20 +156,8 @@ def test_empty_defaults(tmpdir):
 
 
 def test_one_column_defaults(tmpdir):
-    # pylint: disable=abstract-class-instantiated
     tmpdir.chdir()
-    designsheet_df = pd.DataFrame()
-    defaultssheet_df = pd.DataFrame(data=[["foo"]])
-    writer = pd.ExcelWriter("design_matrix.xlsx")
-    designsheet_df.to_excel(
-        writer,
-        sheet_name="DesignSheet01",
-        index=False,
-    )
-    defaultssheet_df.to_excel(
-        writer, sheet_name="DefaultValues", index=False, header=None
-    )
-    writer.save()
+    write_design_xlsx("design_matrix.xlsx", defaultsdf=pd.DataFrame(data=[["foo"]]))
 
     with pytest.raises(SystemExit):
         design2params._read_defaultssheet("design_matrix.xlsx", "DefaultValues")
@@ -191,14 +171,9 @@ def test_three_column_defaults(tmpdir):
     """
     # pylint: disable=abstract-class-instantiated
     tmpdir.chdir()
-    designsheet_df = pd.DataFrame()
-    defaultssheet_df = pd.DataFrame(data=[["foo", "bar", "com"]])
-    writer = pd.ExcelWriter("design_matrix.xlsx")
-    designsheet_df.to_excel(writer, sheet_name="DesignSheet01", index=False)
-    defaultssheet_df.to_excel(
-        writer, sheet_name="DefaultValues", index=False, header=None
+    write_design_xlsx(
+        "design_matrix.xlsx", defaultsdf=pd.DataFrame(data=[["foo", "bar", "com"]])
     )
-    writer.save()
 
     parsed_defaults = design2params._read_defaultssheet(
         "design_matrix.xlsx", "DefaultValues"
@@ -241,16 +216,35 @@ def test_open_excel_file_header_missing(input_data):
         )
 
 
-def test_open_excel_file_value_missing(input_data):
-    with pytest.raises(SystemExit):
-        design2params.run(
-            1,
-            "design_matrix_missing_value.xlsx",
-            "DesignSheet01",
-            "DefaultValues",
-            design2params._PARAMETERS_TXT,
-            log_level=logging.DEBUG,
-        )
+def test_open_excel_file_value_missing(input_data, caplog):
+    """Check that we get SystemExit only for those realizations
+    affected by missing cell values
+
+    (all realizations see warnings for empty cells in other realizations)
+    """
+    reals_with_missing_values = [7, 11]
+    for real in range(0, 25):
+        if real in reals_with_missing_values:
+            with pytest.raises(SystemExit):
+                design2params.run(
+                    real,
+                    "design_matrix_missing_value.xlsx",
+                    "DesignSheet01",
+                    "DefaultValues",
+                    design2params._PARAMETERS_TXT,
+                    log_level=logging.DEBUG,
+                )
+        else:
+            design2params.run(
+                real,
+                "design_matrix_missing_value.xlsx",
+                "DesignSheet01",
+                "DefaultValues",
+                design2params._PARAMETERS_TXT,
+                log_level=logging.DEBUG,
+            )
+    assert "Realization 7, column RMS_SEED" in caplog.text
+    assert "Realization 11, column MULTFLT_F1" in caplog.text
 
 
 @pytest.mark.parametrize(
@@ -269,7 +263,6 @@ def test_open_excel_file_value_missing(input_data):
 def test_existing_parameterstxt(
     exist_params, design_m, expected_warning, tmpdir, caplog
 ):
-    # pylint: disable=abstract-class-instantiated
     """Test warnings emitted when the file parameters.txt is prefilled"""
     tmpdir.chdir()
 
@@ -279,19 +272,13 @@ def test_existing_parameterstxt(
 
     designsheet_df = pd.DataFrame.from_records(data=[design_m])
     designsheet_df.insert(0, "REAL", [0])
-    defaultssheet_df = pd.DataFrame()
-    writer = pd.ExcelWriter("design_matrix.xlsx")
-    designsheet_df.to_excel(writer, sheet_name="DesignSheet01", index=False)
-    defaultssheet_df.to_excel(
-        writer, sheet_name="DefaultValues", index=False, header=None
-    )
-    writer.save()
+
+    write_design_xlsx("design_matrix.xlsx", designdf=designsheet_df)
+
     design2params.run(
         0,
         "design_matrix.xlsx",
-        "DesignSheet01",
-        "DefaultValues",
-        params_file,
+        parametersfilename=params_file,
         log_level=logging.DEBUG,
     )
 
@@ -307,7 +294,7 @@ def test_open_excel_file_wrong_defaults(input_data):
             1,
             "design_matrix_missing_value.xlsx",
             "DesignSheet01",
-            "DefaultValues",
+            "DefaultValuesWRONGNAME",
             design2params._PARAMETERS_TXT,
             log_level=logging.DEBUG,
         )
@@ -339,25 +326,17 @@ def test_open_excel_file_wrong_defaults(input_data):
 def test_single_cell_values(cellvalue, expected_parameters_str, tmpdir):
     """Test how certain single values go through the Excel input sheets
     and all the way to parameters.txt"""
-    # pylint: disable=abstract-class-instantiated
     tmpdir.chdir()
-
-    designsheet_df = pd.DataFrame(columns=["REAL", "SOMEKEY"], data=[[0, cellvalue]])
-
-    defaultssheet_df = pd.DataFrame()
-    writer = pd.ExcelWriter("design_matrix.xlsx")
-    designsheet_df.to_excel(writer, sheet_name="DesignSheet01", index=False)
-    defaultssheet_df.to_excel(
-        writer, sheet_name="DefaultValues", index=False, header=None
+    write_design_xlsx(
+        "design_matrix.xlsx",
+        designdf=pd.DataFrame(columns=["REAL", "SOMEKEY"], data=[[0, cellvalue]]),
     )
-    writer.save()
+
     params_file = "parameters.txt"
     design2params.run(
         0,
         "design_matrix.xlsx",
-        "DesignSheet01",
-        "DefaultValues",
-        params_file,
+        parametersfilename=params_file,
         log_level=logging.DEBUG,
     )
     with open(params_file) as p_file:
@@ -384,36 +363,27 @@ def test_pair_cell_values(cellvalues, expected_parameters_strs, tmpdir):
 
     This is to ensure that differing datatypes in a single Excel columns does
     not affect individual values."""
-    # pylint: disable=abstract-class-instantiated
     tmpdir.chdir()
 
-    designsheet_df = pd.DataFrame(
-        columns=["REAL", "SOMEKEY"], data=[[0, cellvalues[0]], [1, cellvalues[1]]]
+    write_design_xlsx(
+        "design_matrix.xlsx",
+        designdf=pd.DataFrame(
+            columns=["REAL", "SOMEKEY"], data=[[0, cellvalues[0]], [1, cellvalues[1]]]
+        ),
     )
 
-    defaultssheet_df = pd.DataFrame()
-    writer = pd.ExcelWriter("design_matrix.xlsx")
-    designsheet_df.to_excel(writer, sheet_name="DesignSheet01", index=False)
-    defaultssheet_df.to_excel(
-        writer, sheet_name="DefaultValues", index=False, header=None
-    )
-    writer.save()
     params_0 = "parameters0.txt"
     design2params.run(
         0,
         "design_matrix.xlsx",
-        "DesignSheet01",
-        "DefaultValues",
-        params_0,
+        parametersfilename=params_0,
         log_level=logging.DEBUG,
     )
     params_1 = "parameters1.txt"
     design2params.run(
         1,
         "design_matrix.xlsx",
-        "DesignSheet01",
-        "DefaultValues",
-        params_1,
+        parametersfilename=params_1,
         log_level=logging.DEBUG,
     )
     with open(params_0) as p_file:
@@ -424,3 +394,182 @@ def test_pair_cell_values(cellvalues, expected_parameters_strs, tmpdir):
     key_vals_real1 = extract_key_value(params_lines_1)
     assert key_vals_real0["SOMEKEY"] == expected_parameters_strs[0]
     assert key_vals_real1["SOMEKEY"] == expected_parameters_strs[1]
+
+
+@pytest.mark.parametrize(
+    "paramnames",
+    (
+        [
+            ["REAL", "SOMEKEY "],
+            ["REAL", " SOMEKEY"],
+            ["REAL", " SOMEKEY "],
+            ["REAL", "\tSOMEKEY"],
+            ["REAL", "SOMEKEY\t"],
+            ["REAL", "SOMEKEY\n"],
+        ]
+    ),
+)
+def test_headers_trailing_whitespace(paramnames, tmpdir):
+    """Parameter names can have un-noticed trailing whitespace in Excel.
+
+    This can happen both in the design- and the defaultssheet.
+
+    This should error hard as it has no believed use-case and only
+    creates user confusion.
+    """
+    tmpdir.chdir()
+
+    write_design_xlsx(
+        "design_matrix.xlsx",
+        designdf=pd.DataFrame(columns=paramnames, data=[[0, "foo"]]),
+    )
+    write_design_xlsx(
+        "design_matrix_onlydefaults.xlsx",
+        defaultsdf=pd.DataFrame(data=[[paramnames[1], "bar"]]),
+    )
+
+    with pytest.raises(SystemExit, match="whitespace"):
+        design2params.run(
+            0,
+            "design_matrix.xlsx",
+        )
+    if version.parse(pd.__version__) < version.parse("0.25"):
+        expected_error = "Invalid value in design matrix header"
+    else:
+        expected_error = "whitespace"
+    with pytest.raises(SystemExit, match=expected_error):
+        design2params.run(
+            0,
+            "design_matrix_onlydefaults.xlsx",
+            parametersfilename="parameters_onlydefaults.txt",
+        )
+
+
+@pytest.mark.parametrize("paramname", design2params.DENYLIST)
+def test_denylist_designsheet(paramname, tmpdir):
+    """Some parameter names are reserved, ensure we error hard if found."""
+    tmpdir.chdir()
+
+    write_design_xlsx(
+        "design_matrix.xlsx",
+        designdf=pd.DataFrame(columns=["REAL", paramname], data=[[0, "foo"]]),
+    )
+    with pytest.raises(SystemExit, match="not allowed"):
+        design2params.run(
+            0,
+            "design_matrix.xlsx",
+        )
+
+
+@pytest.mark.parametrize("paramname", design2params.DENYLIST_DEFAULTS)
+def test_denylist_defaults(paramname, tmpdir):
+    """Some parameter names are reserved, ensure we error hard if found.
+
+    The defaults sheet can have more denied parameters than the designsheet,
+    in particular the "REAL" column."""
+    tmpdir.chdir()
+    write_design_xlsx(
+        "design_matrix.xlsx", defaultsdf=pd.DataFrame(data=[[paramname, "bar"]])
+    )
+    if version.parse(pd.__version__) < version.parse("0.25"):
+        expected_error = "Invalid value in design matrix header"
+    else:
+        expected_error = "not allowed"
+    with pytest.raises(SystemExit, match=expected_error):
+        design2params.run(
+            0,
+            "design_matrix.xlsx",
+        )
+
+
+@pytest.mark.parametrize("paramname", [1, 1.0])
+def test_invalid_pandas_header_error(tmpdir, paramname):
+    tmpdir.chdir()
+
+    write_design_xlsx(
+        "design_matrix.xlsx",
+        designdf=pd.DataFrame(columns=["REAL", paramname], data=[[0, "foo"]]),
+    )
+
+    expected_error = "Invalid value in design matrix header"
+    with pytest.raises(SystemExit, match=expected_error):
+        design2params.run(
+            0,
+            "design_matrix.xlsx",
+        )
+
+
+@pytest.mark.parametrize("paramname", [1, 1.0])
+def test_denylist_expected_error(paramname, tmpdir):
+    tmpdir.chdir()
+    write_design_xlsx(
+        "design_matrix.xlsx", defaultsdf=pd.DataFrame(data=[[paramname, "bar"]])
+    )
+    if version.parse(pd.__version__) < version.parse("0.25"):
+        expected_error = "Invalid value in design matrix header"
+    else:
+        expected_error = "does not exist in design matrix"
+    with pytest.raises(SystemExit, match=expected_error):
+        design2params.run(
+            0,
+            "design_matrix.xlsx",
+        )
+
+
+@pytest.mark.parametrize(
+    "paramset", [["REAL", "REAL"], ["REAL", "REAL", "REAL"], ["FOO", "FOO"]]
+)
+def test_duplicated_designcolumns(paramset, tmpdir, caplog):
+    """If the excel sheet contains duplicated column headers, pandas will
+    happily read it but modify column names. This is most likely a user error,
+    and a warning is issued"""
+
+    tmpdir.chdir()
+    write_design_xlsx(
+        "design_matrix.xlsx",
+        designdf=pd.DataFrame(
+            columns=["REAL"] + paramset, data=[[0] + ["foo"] * len(paramset)]
+        ),
+    )
+
+    design2params.run(
+        0,
+        "design_matrix.xlsx",
+    )
+    assert "are probably duplicated" in caplog.text
+
+
+def write_design_xlsx(
+    filename,
+    designdf=None,
+    defaultsdf=None,
+    designsheetname="DesignSheet01",
+    defaultssheetname="DefaultValues",
+):
+    """Generate an XLS file with a design sheet and a defaults sheet.
+
+    Args:
+        filename (str)
+        designdf (pd.DataFrame): Design data. Empty data will be written if None
+        defaultsdf (pd.DataFrame): defaults data. Empty data will be written if None
+        designsheetname (str): sheet-name in xlsx file. Defaulted if not provided
+        defaultssheetname (str): sheet-name in xlsx file. Defaulted if not provided
+    """
+    # pylint: disable=abstract-class-instantiated
+    writer = pd.ExcelWriter(filename)
+    if designdf is not None:
+        designdf.to_excel(writer, sheet_name=designsheetname, index=False)
+    else:
+        # Write empty sheet.
+        pd.DataFrame().to_excel(writer, sheet_name=designsheetname, index=False)
+
+    if defaultsdf is not None:
+        defaultsdf.to_excel(
+            writer, sheet_name=defaultssheetname, index=False, header=None
+        )
+    else:
+        # Write empty sheet.
+        pd.DataFrame().to_excel(
+            writer, sheet_name=defaultssheetname, index=False, header=None
+        )
+    writer.save()
